package org.esupportail.cas.server.handlers.ldap;

import org.dom4j.Element;
import org.esupportail.cas.server.util.MisconfiguredHandlerException;

/**
 * This class implements a bind LDAP handler class. It is used by
 * GenericHandler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public final class BindLdapHandler extends LdapHandler {
	
	/**
	 * the base for LDAP searches.
	 */
	private String searchBase;
	
	/**
	 * the scope for LDAP searches.
	 */
	private String scope;
	
	/**
	 * the DN to bind to the LDAP directory.
	 */
	private String bindDn;
	
	/**
	 * the password to bind to the LDAP directory.
	 */
	private String bindPassword;

	/**
	 * A flag set to true if multiple accounts in the LDAP directory are allowed.
	 */
	private boolean multipleAccountsEnabled;
	
	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file 
	 * @param configDebug debugging mode of the global configuration
	 * @throws Exception Exception
	 */
	public BindLdapHandler(
			final Element handlerElement, 
			final Boolean configDebug) throws Exception {
		super(handlerElement, configDebug);
		traceBegin();

		searchBase = getConfigSubElementContent("search_base", true/*needed*/);
		trace("search_base = " + searchBase);

		scope = getConfigSubElementContent("scope", true/*needed*/);
		trace("scope = " + scope);

		bindDn = getConfigSubElementContent("bind_dn", false/*not needed*/);
		if (bindDn.equals("")) {
			trace("An anonymous connection to the LDAP directory will be used.");
			bindPassword = "";
		} else {
			bindPassword = getConfigSubElementContent("bind_password", false/*not needed*/);
		}
		trace("bind_dn = " + bindDn);
		trace("bind_password = " + bindPassword);

		boolean enableMultipleAccounts = hasConfigSubElement("enable_multiple_accounts");
		boolean disableMultipleAccounts = hasConfigSubElement("disable_multiple_accounts");
		
		if (enableMultipleAccounts && disableMultipleAccounts) {
			traceThrow(new MisconfiguredHandlerException(
					"enable_multiple_accounts and disable_multiple_accounts "
					+ "tags can not be used simultaneously"));
		}
		multipleAccountsEnabled = false;
		if (enableMultipleAccounts) {
			multipleAccountsEnabled = true;
		}
		trace("enable_multiple_accounts = " + multipleAccountsEnabled);
		
		// add the LDAP servers
		addServers(true/*serverElementNeeded*/, getClass().getPackage().getName() + ".BindLdapServer");

		traceEnd();
	}
	
	/**
	 * Retrieve the base for LDAP searches.
	 *
	 * @return a string.
	 */
	String getSearchBase() {
		return searchBase;
	}
	
	/**
	 * Retrieve the scope for LDAP searches.
	 *
	 * @return a string.
	 */
	String getScope() {
		return scope;
	}
	
	/**
	 * Retrieve the DN to bind to the LDAP directory.
	 *
	 * @return a string.
	 */
	String getBindDn() {
		return bindDn;
	}
	
	/**
	 * Retrieve the password to bind to the LDAP directory.
	 *
	 * @return a string.
	 */
	String getBindPassword() {
		return bindPassword;
	}
	
	/**
	 * Tell if multiple accounts in the LDAP directory are allowed.
	 * 
	 * @return a boolean.
	 */
	protected boolean areMultipleAccountsEnabled() {
		return multipleAccountsEnabled;
	}
}
