package org.esupportail.cas.server.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.dom4j.Element;

/**
 * This abstract class implements a redundant handler, with a list 
 * of servers that will be used for redundancy.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public abstract class RedundantHandler extends BasicHandler {

	/**
	 * the list of servers to be used for authentication. Modified by
	 * addServer().
	 */
	private LinkedList servers;

	
	/**
	 * retrieve the list of the servers.
	 *
	 * @return the list of the servers used for authentication.
	 */
	public final List getServers() {
		return servers;
	}
	
	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file
	 * @param configDebug debugging mode of the global configuration
	 */
	protected RedundantHandler(
			final Element handlerElement, 
			final Boolean configDebug) {
		super(handlerElement, configDebug);
		traceBegin();
		servers = new LinkedList();
		traceEnd();
	}
	
	/**
	 * Constructor.
	 *
	 * @param serverElementNeeded true to force the handler to have at least one server sub-element
	 * @param serverClassname the classname of the server to create for the handler
	 * @throws Exception Exception
	 */	
	protected final void addServers(
			final boolean serverElementNeeded,
			final String serverClassname) throws Exception {
		traceBegin();

		Element configElement = getConfigElement();
		
		// check if a config/server element is needed
		if (configElement == null) {
			if (serverElementNeeded) {
				traceThrow(new MisconfiguredHandlerException(
						"A \"config\" element is needed by \"" 
						+ getClass().getName() 
						+ "\" handlers."));
			}
			return;
		}

		trace("Looking for servers...");

		// get the list of all the server elements		
		List serverElements = configElement.elements("server");
		
		// check that at least one server is declared 
		if (serverElementNeeded) {
			if (serverElements.isEmpty()) {
				traceThrow(new MisconfiguredHandlerException(
						"At least one \"server\" element is needed to configure \"" 
						+ getClass().getName() 
						+ "\" handlers."));
			}
		}

		for (Iterator i = serverElements.iterator(); i.hasNext();) {
			trace("Found a server.");

			// load the server class
			Class serverClass = null;
			try {
				trace("Getting the " + serverClassname + " class...");
				serverClass = Class.forName(serverClassname);
			} catch (ClassNotFoundException e) {
				// this should never happen
				traceThrow(new MisconfiguredHandlerException(
						"Class \"" 
						+ serverClassname 
						+ "\" could not be loaded."));
			}

			// get the constructor
			Class[] serverConstructorArgumentTypes = {
					Boolean.class, 
					RedundantHandler.class, 
					Element.class};
			Constructor serverConstructor = null;
			try {
				trace("Getting the constructor...");
				serverConstructor = serverClass.getConstructor(serverConstructorArgumentTypes);
			} catch (Exception e) {
				traceThrow(new MisconfiguredHandlerException(
						"The constructor of Class \"" 
						+ serverClassname 
						+ "\" could not be loaded because a \"" 
						+ e.getClass().getName() 
						+ "\" exception was raised: " 
						+ e.getMessage()));
			}

			// create an instance
			Object[] serverConstructorArguments = {new Boolean(isDebug()), this, i.next()};
			Server server = null;
			try {
				trace("Creating a new instance...");
				server = (Server) serverConstructor.newInstance(serverConstructorArguments);
			} catch (InvocationTargetException e) {
				traceThrow(new MisconfiguredHandlerException(e.getCause().getMessage()));
			} catch (Exception e) {
				traceThrow(new MisconfiguredHandlerException(
						"Class \"" 
						+ serverClassname 
						+ "\" could not be instanciated because a \"" 
						+ e.getClass().getName() 
						+ "\" exception was raised: " 
						+ e.getMessage()));
			}

			// add the newly created object to the list of servers
			servers.add(server);
			
			trace("Server added.");
		}

		trace(servers.size() +  " servers found.");
		traceEnd();
	}
	
	/**
	 * Tries to Authenticate a user by accessing all the servers.
	 *
	 * @param username the username to authenticate
	 * @param password the correspoding password
	 *
	 * @return BasicHandlerSUCCEDED on success, or 
	 * BasicHandler.FAILED_CONTINUE otherwise.
	 */
	 public final int authenticate(final String username,
			final String password) {
		traceBegin();
		
		for (Iterator i = servers.iterator(); i.hasNext();) {
			Server server = (Server) i.next();
			switch (server.authenticate(username, password)) {
				case Server.AUTHENTICATE_SUCCESS:
					trace("Server matched.");
				traceEnd("SUCCEEDED");
				return SUCCEEDED;
				case Server.AUTHENTICATE_NOAUTH:
					trace("Server did not match.");
				traceEnd("FAILED_CONTINUE");
				return FAILED_CONTINUE;
				default:
					trace("Server failure, trying next");
				break;
			}	    
		}
		trace("No server matched.");
		traceEnd("FAILED_CONTINUE");
		return FAILED_CONTINUE;
	}
	
}

